@@ -1,6 +1,6 @@
 /* Test file for mpfr_sqrt_ui.
 
-Copyright 2000-2003, 2006-2017 Free Software Foundation, Inc.
+Copyright 2000-2003, 2006-2018 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -20,9 +20,6 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 static void
@@ -37,7 +34,8 @@ check (unsigned long a, mpfr_rnd_t rnd_mode, const char *qs)
       printf ("mpfr_sqrt_ui failed for a=%lu, rnd_mode=%s\n",
               a, mpfr_print_rnd_mode (rnd_mode));
       printf ("sqrt gives %s, mpfr_sqrt_ui gives ", qs);
-      mpfr_out_str(stdout, 10, 0, q, MPFR_RNDN);
+      mpfr_out_str (stdout, 10, 0, q, MPFR_RNDN);
+      putchar ('\n');
       exit (1);
     }
   mpfr_clear (q);
